PROTO = {
    'spaceone.dashboard.interface.grpc.v1.domain_dashboard': ['DomainDashboard'],
    'spaceone.dashboard.interface.grpc.v1.project_dashboard': ['ProjectDashboard'],
    'spaceone.dashboard.interface.grpc.v1.custom_widget': ['CustomWidget']
}
